/* example-start helloworld helloworld.c */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

static GtkWidget *pixmapArea;
static GtkWidget *localButton;
static GtkWidget *window;
static GtkStyle  *style;
static int curSelection;

int checkInPath(const char *filename)
{
  gchar *pathStr = g_strdup(getenv("PATH"));
  gchar *accessStr;
  gchar *p;

  while ((p = strrchr(pathStr, ':')) != NULL) {
    *p = '\0';
    /*g_print("curdir is %s, filename is %s\n",p+1, filename);*/
    accessStr = g_strdup_printf("%s/%s",p+1, filename);
    if (access(accessStr, X_OK) == 0) {
      free(accessStr);
      return 1;
    }
    free(accessStr);
  }
  /*g_print("curdir is %s, filename is %s\n",pathStr, filename);*/
  accessStr = g_strdup_printf("%s/%s",pathStr, filename);
  if (access(accessStr, X_OK) == 0) {
    free(accessStr);
    return 1;
  }

  free(accessStr);
  return 0;
}

gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  return (FALSE);
}

/* another callback */
void destroy (GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
}

void accept(GtkWidget *widget, gpointer data)
{
  gchar *exeStr=0;
  GtkWidget *infoWin;
  GtkWidget *vbox, *hbox, *infoLabel, *okButton;

  switch(curSelection) {
  case 0: // GNOME
    exeStr = g_strdup_printf("switchdesk-helper gnome %s",
			     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(localButton)) ? "local" : "");
    break;
  case 1: // KDE
    exeStr = g_strdup_printf("switchdesk-helper kde %s",
			     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(localButton)) ? "local" : "");
    break;
  case 2: // Another Level
    exeStr = g_strdup_printf("switchdesk-helper anotherlevel %s",
			     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(localButton)) ? "local" : "");
    break;
  case 3: // Revert
    exeStr = g_strdup_printf("switchdesk-helper revert %s",
			     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(localButton)) ? "local" : "");
    break;
  }

  system(exeStr);
  free(exeStr);

  gtk_widget_hide(widget);

  infoWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  //  gtk_window_set_modal(GTK_WINDOW(infoWin), 1);
  
  gtk_window_set_title(GTK_WINDOW(infoWin), "Desktop Switcher");          
  gtk_signal_connect (GTK_OBJECT (infoWin), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  
  gtk_signal_connect (GTK_OBJECT (infoWin), "destroy",
		      GTK_SIGNAL_FUNC (destroy), NULL);
          
  gtk_container_border_width (GTK_CONTAINER (infoWin), 10);
  gtk_widget_show (infoWin);

  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(infoWin), vbox);

  infoLabel = gtk_label_new("Your desktop configuration has successfully been updated,\n"
			    "but you must restart the X Window System for the changes\n"
			    "to take effect.");
  gtk_label_set_justify(GTK_LABEL(infoLabel), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start(GTK_BOX(vbox), infoLabel, FALSE, FALSE, 0);
  gtk_widget_show(infoLabel);

  hbox = gtk_hbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(vbox), hbox);
  
  okButton = gtk_button_new_with_label("     OK     ");
  gtk_signal_connect (GTK_OBJECT (okButton), "clicked",
		      GTK_SIGNAL_FUNC (gtk_main_quit), infoWin);
  gtk_box_pack_start(GTK_BOX(hbox), okButton, TRUE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS(okButton, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(GTK_WIDGET(okButton));
  gtk_widget_show(okButton);
			    
  gtk_widget_show(hbox);
  gtk_widget_show(vbox);
}

void reject(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(widget);
}

void updatePic(GtkWidget *widget, gpointer num)
{
  GdkPixmap *p=0;
  int *i;

  i = (int *) num;

  switch(*i) {
  case 0:
    p = gdk_pixmap_create_from_xpm(window->window, NULL, 
				   &style->bg[GTK_STATE_NORMAL],
				   "gnome.xpm");
    curSelection = 0;
    break;
  case 1:
    p = gdk_pixmap_create_from_xpm(window->window, NULL, 
				   &style->bg[GTK_STATE_NORMAL],
				   "kde.xpm");
    curSelection = 1;
    break;
  case 2:
    p = gdk_pixmap_create_from_xpm(window->window, NULL, 
				   &style->bg[GTK_STATE_NORMAL],
				   "anotherlevel.xpm");
    curSelection = 2;
    break;
  case 3:
    p = gdk_pixmap_create_from_xpm(window->window, NULL, 
				   &style->bg[GTK_STATE_NORMAL],
				   "question.xpm");
    curSelection = 3;
    break;
  }

  gtk_pixmap_set(GTK_PIXMAP(pixmapArea), p, NULL);
  gdk_pixmap_unref(p);
}

int main (int argc, char *argv[])
{
  /* GtkWidget is the storage type for widgets */
  GtkWidget *vbox=0, *hbox1=0, *hbox2=0, *topVBox=0;
  GtkWidget *groupFrame=0;
  GtkWidget *gnomeButton=0, *kdeButton=0, *alButton=0, 
    *revertButton=0, *prevButton=0;
  GtkWidget *okButton=0, *cancelButton=0;
  GSList *group=0;
  gchar *dispStr=0;
  GtkWidget *dispLabel=0;
  GdkPixmap *gdkpm=0;
  int gnomearg=0, kdearg=1, alarg=2, revertarg=3;

  /* this is called in all GTK applications.  arguments are parsed from
   * the command line and are returned to the application. */
  gtk_init (&argc, &argv);

  chdir("/usr/share/apps/switchdesk/pics");

  /* create a new window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_title(GTK_WINDOW(window), "Desktop Switcher");          
  /* when the window is given the "delete_event" signal (this is given
   * by the window manager, usually by the 'close' option, or on the
   * titlebar), we ask it to call the delete_event () function
   * as defined above.  The data passed to the callback
   * function is NULL and is ignored in the callback function. */
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  
  /* here we connect the "destroy" event to a signal handler.  
   * This event occurs when we call gtk_widget_destroy() on the window,
   * or if we return 'FALSE' in the "delete_event" callback. */
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC (destroy), NULL);
          
  /* sets the border width of the window. */
  gtk_container_border_width (GTK_CONTAINER (window), 10);

  /* and the window */
  gtk_widget_show (window);

  topVBox = gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(window), topVBox);


  dispStr = g_strdup_printf("Current display is ");
  if (*getenv("DISPLAY") == ':')
    dispStr = g_strconcat(dispStr, getenv("HOSTNAME"), NULL);
  dispStr = g_strconcat(dispStr, getenv("DISPLAY"), NULL);
  dispLabel = gtk_label_new(dispStr);
  gtk_box_pack_start(GTK_BOX(topVBox), dispLabel, TRUE, TRUE, 0);
  gtk_widget_show(dispLabel);

  groupFrame = gtk_frame_new("Available Desktops");
  gtk_box_pack_start(GTK_BOX(topVBox), groupFrame, TRUE, TRUE, 0);
  gtk_widget_show(groupFrame);

  hbox1 = gtk_hbox_new(FALSE, 10);
  gtk_container_border_width (GTK_CONTAINER(hbox1), 10);
  gtk_container_add(GTK_CONTAINER(groupFrame), hbox1);

  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(hbox1), vbox);
  
  if (checkInPath("panel")) {
    gnomeButton = gtk_radio_button_new_with_label(NULL, "GNOME");
    gtk_signal_connect (GTK_OBJECT (gnomeButton), "clicked",
			GTK_SIGNAL_FUNC (updatePic), (gpointer) &gnomearg);  
    /* this packs the button into the window (a gtk container). */
    gtk_box_pack_start (GTK_BOX(vbox), gnomeButton, TRUE, TRUE, 0);
    gtk_widget_show (gnomeButton);
    prevButton = gnomeButton;
  }

  if (prevButton)
    group = gtk_radio_button_group(GTK_RADIO_BUTTON(prevButton));
  if (checkInPath("kpanel")) {
    kdeButton = gtk_radio_button_new_with_label(group, "KDE");
    gtk_signal_connect(GTK_OBJECT(kdeButton), "clicked",
		       GTK_SIGNAL_FUNC(updatePic), (gpointer) &kdearg);
    gtk_box_pack_start(GTK_BOX(vbox), kdeButton, TRUE, TRUE, 0);
    gtk_widget_show(kdeButton);
    prevButton = kdeButton;
  }

  if (prevButton)
    group = gtk_radio_button_group(GTK_RADIO_BUTTON(prevButton));
  alButton = gtk_radio_button_new_with_label(group, "Another Level");
  gtk_signal_connect(GTK_OBJECT(alButton), "clicked",
		     GTK_SIGNAL_FUNC(updatePic), (gpointer) &alarg);
  gtk_box_pack_start(GTK_BOX(vbox), alButton, TRUE, TRUE, 0);
  gtk_widget_show(alButton);
  prevButton = alButton;

  if (prevButton)
    group = gtk_radio_button_group(GTK_RADIO_BUTTON(prevButton));
  if (access(g_strdup_printf("%s/.Xclients.switchdesk",getenv("HOME")),
	     F_OK) == 0) {
    revertButton = gtk_radio_button_new_with_label(group, 
						   "Revert to user settings");
    gtk_signal_connect(GTK_OBJECT(revertButton), "clicked",
		       GTK_SIGNAL_FUNC(updatePic), (gpointer) &revertarg);
    gtk_box_pack_start(GTK_BOX(vbox), revertButton, TRUE, TRUE, 0);
    gtk_widget_show(revertButton);
  }
    
  gtk_widget_show(vbox);

  localButton = gtk_check_button_new_with_label("Change only applies to current display");
  gtk_box_pack_start(GTK_BOX(topVBox), localButton, TRUE, TRUE, 0);
  gtk_widget_show(localButton);

  hbox2 = gtk_hbox_new(TRUE, 10);
  gtk_container_add(GTK_CONTAINER(topVBox), hbox2);

  okButton = gtk_button_new_with_label("OK");
  /* This will cause the window to be destroyed by calling
   * gtk_widget_destroy(window) when "clicked".  Again, the destroy
   * signal could come from here, or the window manager. */
  gtk_signal_connect_object (GTK_OBJECT (okButton), "clicked",
			     GTK_SIGNAL_FUNC (accept),
			     GTK_OBJECT (window));
          
  gtk_box_pack_start(GTK_BOX(hbox2), okButton, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(okButton, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(GTK_WIDGET(okButton));
  gtk_widget_show(okButton);

  cancelButton = gtk_button_new_with_label("Cancel");
  /* This will cause the window to be destroyed by calling
   * gtk_widget_destroy(window) when "clicked".  Again, the destroy
   * signal could come from here, or the window manager. */
  gtk_signal_connect_object (GTK_OBJECT (cancelButton), "clicked",
			     GTK_SIGNAL_FUNC (reject),
			     GTK_OBJECT (window));
          
  gtk_box_pack_start(GTK_BOX(hbox2), cancelButton, TRUE, TRUE, 0);
  gtk_widget_show(cancelButton);

  gtk_widget_show(hbox2);

  /* set up pixmaps */
  style = gtk_widget_get_style(window);

  gdkpm = gdk_pixmap_create_from_xpm(window->window, NULL, 
				     &style->bg[GTK_STATE_NORMAL],
				     "gnome.xpm");

  pixmapArea = gtk_pixmap_new(gdkpm, NULL);
  gdk_pixmap_unref(gdkpm);
  gtk_box_pack_start(GTK_BOX(hbox1), pixmapArea, TRUE, TRUE, 0);
  gtk_widget_show(pixmapArea);

  gtk_widget_show(hbox1);
  
  //  gtk_pixmap_set(GTK_PIXMAP(pixmapArea), gdkpm, NULL);

  gtk_widget_show(topVBox);

  if (gnomeButton) {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gnomeButton), 1);
    updatePic(NULL, (gpointer) &gnomearg);
  } else if (kdeButton) {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(kdeButton), 1);
    updatePic(NULL, (gpointer) &kdearg);
  } else {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(alButton), 1);
    updatePic(NULL, (gpointer) &alarg);
  }

  /* all GTK applications must have a gtk_main(). Control ends here
   * and waits for an event to occur (like a key press or mouse event). */
  gtk_main ();
          
  return 0;
}
