// Desktop Switching Tool
// (c) 1999 Red Hat Software, Inc.

#include <stdlib.h>
#include <unistd.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <kbuttonbox.h>
#include <kapp.h>
#include <kiconloader.h>

#include "switchdesk.h"
#include "switchdesk.moc"

bool checkInPath(const char *filename)
{
  QString pathStr = getenv("PATH");
  QString curDir;
  QString accessStr;
  int i;
  while ((i = pathStr.findRev(":")) != -1) {
    curDir = pathStr.right(pathStr.length() - (i+1)).copy();
    pathStr.truncate(i);
    accessStr = curDir.data();
    accessStr += "/";
    accessStr += filename;
    if (access(accessStr.data(), X_OK) == 0) {
      return TRUE;
    }
  }
  accessStr = pathStr.data();
  accessStr += "/";
  accessStr += filename;
  if (access(accessStr.data(), X_OK) == 0) {
    return TRUE;
  }

  return FALSE;
}

SDDialog::SDDialog(QWidget *parent, const char *name)
  : QDialog(parent, name)
{

  gnomeButton = kdeButton = alButton = 0L;

  setCaption("Desktop Switcher");

  QBoxLayout *layout = new QVBoxLayout(this, 10);

  QString dispStr = i18n("Current display is ");
  if (strlen(getenv("DISPLAY")) != 0) {
    if (*getenv("DISPLAY") == ':')
      dispStr += getenv("HOSTNAME");
    dispStr += getenv("DISPLAY");
    QLabel *dispLabel = new QLabel(dispStr.data(), this);
    dispLabel->setMinimumSize(dispLabel->sizeHint());
    layout->addWidget(dispLabel);
  }

  QButtonGroup *gbox = new QButtonGroup(i18n("Available Desktops"), this);
  layout->addWidget(gbox);

  QBoxLayout *subLayout = new QHBoxLayout(gbox, 10);
  QBoxLayout *subLayout2 = new QVBoxLayout();
  subLayout->addLayout(subLayout2);
  subLayout2->addSpacing(10);

  if (checkInPath("panel")) {
    gnomeButton = new QRadioButton("GNOME", gbox);
    gnomeButton->setMinimumSize(gnomeButton->sizeHint());
    connect(gnomeButton, SIGNAL(clicked()), SLOT(updateGnomePic()));
    subLayout2->addWidget(gnomeButton);
  }

  if (checkInPath("kpanel")) {
    kdeButton = new QRadioButton("KDE", gbox);
    kdeButton->setMinimumSize(kdeButton->sizeHint());
    subLayout2->addWidget(kdeButton);
    connect(kdeButton, SIGNAL(clicked()), SLOT(updateKDEPic()));
  }

  alButton = new QRadioButton("AnotherLevel", gbox);
  alButton->setMinimumSize(alButton->sizeHint());
  subLayout2->addWidget(alButton);
  connect(alButton, SIGNAL(clicked()), SLOT(updateALPic()));

  QString oldFile = getenv("HOME");
  oldFile += "/.Xclients.switchdesk";
  if (QFile::exists(oldFile.data())) {
    revertButton = new QRadioButton(i18n("Revert to user settings"), gbox);
    revertButton->setMinimumSize(revertButton->sizeHint());
    QToolTip::add(revertButton, i18n("If you previously had your own custom\n"
				     ".Xclients file, it will be restored from\n"
				     "a backup."));
    subLayout2->addWidget(revertButton);
  }

  subLayout->addSpacing(10);

  picLabel = new QLabel(gbox);
  picLabel->setMinimumSize(QSize(100, 100));
  subLayout->addWidget(picLabel);

  localBox = new QCheckBox(this);
  localBox->setText(i18n("Change only applies to current display"));
  localBox->setMinimumSize(localBox->sizeHint());
  QToolTip::add(localBox, i18n("You may not want your new settings to apply\n"
			       "globally, but only to this current computer.\n"
			       "(Your home directory may be NFS mounted, or\n"
			       "you might be sitting at a slow computer, etc.)"));

  layout->addWidget(localBox);

  /* QFrame *hLine = new QFrame(this);
  hLine->setFrameStyle(QFrame::HLine|QFrame::Sunken);
  hLine->setFixedHeight(hLine->frameWidth());
  layout->addWidget(hLine);*/

  KButtonBox *buttonBox = new KButtonBox(this);
  layout->addWidget(buttonBox);

  buttonBox->addStretch();
  okButton = buttonBox->addButton(i18n("&OK"));
  okButton->setDefault(TRUE);
  connect(okButton, SIGNAL(clicked()), SLOT(accept()));

  cancelButton = buttonBox->addButton(i18n("&Cancel"));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));

  buttonBox->setFixedHeight(buttonBox->sizeHint().height());
  buttonBox->layout();

  subLayout->activate();
  layout->activate();

  resize(sizeHint());

  if (gnomeButton) {
    gnomeButton->setChecked(TRUE); // this should reflect current setting
    updateGnomePic();
  } else if (kdeButton) {
    kdeButton->setChecked(TRUE);
    updateKDEPic();
  } else {
    alButton->setChecked(TRUE);
    updateALPic();
  }

}

SDDialog::~SDDialog()
{
}

void SDDialog::accept()
{
  QString exeStr;

  exeStr = "switchdesk-helper ";
  switch (curSelection) {
  case 0: // GNOME
    exeStr += "gnome";
    break;
  case 1: // KDE
    exeStr += "kde";
    break;
  case 2: // AnotherLevel
    exeStr += "anotherlevel";
    break;
  case 3: // Revert
    exeStr += "revert";
    break;
  }

  if (localBox->isChecked())
    exeStr += " local";

  // actually perform the switch
  system(exeStr.data());

  hide();
  QMessageBox::information(0, i18n("Desktop Switcher"),
			   i18n("Your desktop configuration has successfully been updated,\n"
				"but you must restart the X Window System for the changes\n"
				"to take effect."),
			   i18n("OK"));
  close();
}

void SDDialog::reject()
{
  close();
}

void SDDialog::updatePic(int num)
{
  curSelection = num;
  QPixmap p;

  switch(num) {
  case 0:
    p = Icon("gnome.xpm");
    break;
  case 1:
    p = Icon("kde.xpm");
    break;
  case 2:
    p = Icon("anotherlevel.xpm");
    break;
  case 3:
    p = Icon("question.xpm");
    break;
  }
  picLabel->setPixmap(p);

}

